#include <PalmOS.h>
#include <VFSMgr.h>
#include "MyInstall.h"
#include "AutoInstall.h"
#include "MyInstall_Res.h"


UInt16 launchCode;

typedef struct {
    Boolean active;
} PreferenceType;


MemHandle panelIDsH = NULL;
SysDBListItemType* panelData;
Char** panelNames = NULL;
UInt16 panelCount = 0;

typedef struct {
    RectangleType outerRect;
    RectangleType innerRect;
    void*         savedBits;
} MessageBox;


UInt32 InitMessages( void )
{
    Err         err;
    MessageBox* b;

    b = MemPtrNew( sizeof( MessageBox ) );

    if ( b == NULL )
        return 0;

    b->outerRect.topLeft.x = 5;
    b->outerRect.topLeft.y = 70;
    b->outerRect.extent.x  = 150;
    b->outerRect.extent.y  = 20;

    b->savedBits = WinSaveBits( &( b->outerRect ), &err );

    if ( err != errNone ) {
        MemPtrFree( b );
        return 0;
    }

    b->innerRect.topLeft.x = b->outerRect.topLeft.x + 2;
    b->innerRect.topLeft.y = b->outerRect.topLeft.y + 2;
    b->innerRect.extent.x = b->outerRect.extent.x - 4;
    b->innerRect.extent.y = b->outerRect.extent.y - 4;
    WinEraseRectangle( &( b->innerRect ), 0 );
    WinDrawRectangleFrame( boldRoundFrame, &( b->innerRect ) );
    b->innerRect.topLeft.x = b->outerRect.topLeft.x + 4;
    b->innerRect.topLeft.y = b->outerRect.topLeft.y + 4;
    b->innerRect.extent.x = b->outerRect.extent.x - 8;
    b->innerRect.extent.y = b->outerRect.extent.y - 8;

    return ( UInt32 )b;
}


void CloseMessages( UInt32 cookie )
{
    MessageBox* b;

    if ( cookie == 0 )
        return;

    b = ( MessageBox* )cookie;

    WinRestoreBits( b->savedBits, b->outerRect.topLeft.x, b->outerRect.topLeft.y );

    MemPtrFree( b );
}


void Message( const Char* s, UInt32 cookie )
{
    MessageBox* b;
    Coord       x;
    Coord       y;
    FontID      oldFont;
    RectangleType oldClip;
    RectangleType clip;

    if ( cookie == 0 )
        return;

    b = ( MessageBox* )cookie;

    WinEraseRectangle( &( b->innerRect ), 0 );

    oldFont = FntSetFont( boldFont );

    x    = b->innerRect.topLeft.x + b->innerRect.extent.x / 2 - FntCharsWidth( s, StrLen( s ) ) / 2;
    y    = b->innerRect.topLeft.y + b->innerRect.extent.y / 2 - FntLineHeight() / 2;

    if ( x < b->innerRect.topLeft.x + 2 )
        x = b->innerRect.topLeft.x + 2;

    clip = b->innerRect;

    clip.topLeft.x += 2;
    clip.extent.x  -= 4;

    WinGetClip( &oldClip );
    WinSetClip( &clip );

    WinDrawChars( s, StrLen( s ), x, y );

    WinSetClip( &oldClip );

    FntSetFont( oldFont );
}



static void InstallResident( Boolean state );


PreferenceType pref;


UInt16 GetObjectIndex( UInt16 id )
{
    return FrmGetObjectIndex( FrmGetActiveForm(), id );
}


void* GetObjectPtr( UInt16 id )
{
    return FrmGetObjectPtr( FrmGetActiveForm(), GetObjectIndex( id ) );
}



void Show( UInt16 id, Boolean state )
{
    UInt16 index = GetObjectIndex( id );

    if ( state )
        FrmShowObject( FrmGetActiveForm(), index );
    else
        FrmHideObject( FrmGetActiveForm(), index );
    if ( FrmGetObjectType( FrmGetActiveForm(), index ) == frmControlObj )
        CtlSetUsable( GetObjectPtr( id ), state);
}


void LoadPrefs0( PreferenceType* p )
{
    UInt16 size;

    MemSet( p, sizeof( *p ), 0 );

    p->active = true;

    size = 0;

    if ( noPreferenceFound ==
             PrefGetAppPreferences( appCreator, PREF_ID_MAIN, NULL, &size, true ) ) {
        return;
    }

    if ( sizeof( *p ) < size ) {
        size = sizeof( *p );
    }

    PrefGetAppPreferences( appCreator, PREF_ID_MAIN, p, &size, true );
}



void SavePrefs0( PreferenceType* p )
{
    PrefSetAppPreferences( appCreator, PREF_ID_MAIN, 0, p,
        sizeof( PreferenceType ), true );
}



static Boolean Handler_frmMain( EventType* event )
{
    Boolean handled;
    handled = false;

    switch (event->eType) {
        case winEnterEvent:
            handled = false;
            break;

        case frmOpenEvent: {
            MemHandle handle;
            MemHandle infoHandle;

            CtlSetValue( GetObjectPtr( btnOn ), pref.active );
            CtlSetValue( GetObjectPtr( btnOff ), ! pref.active );

            infoHandle = DmGetResource( 'tSTR', strInfo );

            handle     = MemHandleNew( MemHandleSize( infoHandle ) );

            if ( handle != NULL ) {
                Char* p;
                Char* q;

                p = MemHandleLock( infoHandle );
                q = MemHandleLock( handle );

                MemMove( q, p, MemHandleSize( infoHandle ) );
                MemHandleUnlock( handle );
                MemHandleUnlock( infoHandle );
                DmReleaseResource( infoHandle );

                FldSetTextHandle( GetObjectPtr( fldInfo ), handle );
            }

            FrmDrawForm( FrmGetActiveForm() );

            handled = true;
            break;
        }

        case ctlSelectEvent: {
            Boolean value;

            value = !! CtlGetValue( GetObjectPtr( event->data.ctlEnter.controlID ) );

            switch( event->data.ctlEnter.controlID ) {

                case btnOn:
                    if ( pref.active != value ) {
                        pref.active = value;
                        InstallResident( pref.active );
                        SavePrefs0( &pref );
                    }
                    break;
                case btnOff:
                    if ( pref.active != ! value ) {
                        pref.active = ! value;
                        InstallResident( pref.active );
                        SavePrefs0( &pref );
                    }
                    break;
                case btnGo: {               
                    UInt32 cookie;
                    cookie = InitMessages();
                    CopyFiles( ALL_VOLUMES, Message, cookie, NULL, NULL );
                    CloseMessages( cookie );
                }
            }
            break;
        }
        default:
            break;
    }

    return handled;
}



static void EventLoop( void )
{
    EventType event;
    Boolean   handled;

    do {
       	EvtGetEvent( &event, evtWaitForever );

       	handled = SysHandleEvent( &event );

       	if ( ! handled ) {
            switch ( event.eType ) {
                case frmLoadEvent: {
                    UInt16    formID;
                    FormType* form;

                    formID = event.data.frmLoad.formID;
                    form = FrmInitForm( formID );

                    FrmSetActiveForm( form );

                    switch ( formID ) {
                        case frmMain:
                            FrmSetEventHandler( form, Handler_frmMain );
                            break;
                    }
                    handled = true;
                }
                default:
                    handled = FrmDispatchEvent( &event );
            }
       	}
    } while ( event.eType != appStopEvent );

    FrmCloseAllForms();
}



static Boolean ApplicationStart( void )
{
    LoadPrefs0( &pref );

    InstallResident( pref.active );

    FrmGotoForm( frmMain );

    return true;
}



static void ApplicationStop( void )
{
#ifndef APP
    ClearPanelList();
#endif
}



static Boolean FindMe( UInt16* card, LocalID* id )
{
    DmSearchStateType state;

    return errNone == DmGetNextDatabaseByTypeCreator( true, &state,
               appType, appCreator,
               true, card, id );
}



static void InstallResident( Boolean state )
{
    UInt16 card;
    LocalID id;

    if ( ! FindMe( &card, &id ) )
        return;

    if ( state ) {
        SysNotifyRegister( card, id, sysNotifyVolumeMountedEvent,
            NULL, -0x60, NULL );
    }
    else {
        SysNotifyUnregister( card, id, sysNotifyVolumeMountedEvent, -0x60 );
    }
}



void HandleNotification( SysNotifyParamType* np )
{
    switch ( np->notifyType ) {
        case sysNotifyVolumeMountedEvent: {
            UInt32 cookie;
            
            cookie = InitMessages();
            CopyFiles( ( ( VFSSlotMountParamType* )( np->notifyDetailsP ) )->vfsMountParam.volRefNum,
                Message, cookie, NULL, NULL );
            CloseMessages( cookie );
        }
    }
}


UInt32 PilotMain( UInt16 command, void* cmdPBP, UInt16 flags )
{
    switch ( command ) {
        case sysAppLaunchCmdPanelCalledFromApp:
        case sysAppLaunchCmdNormalLaunch:
        {
            Err err;

            launchCode = command;

            ErrTry {
                err = errNone;
                if ( ApplicationStart() )
                    EventLoop();
                ApplicationStop();
            }
            ErrCatch( err ) {
                ApplicationStop();
                if ( err != errNone )
                    return err;
                break;
            } ErrEndCatch

            break;
        }

        case sysAppLaunchCmdSystemReset:
        case sysAppLaunchCmdSyncNotify: {
            PreferenceType p;

            LoadPrefs0( &p );

            InstallResident( p.active );
            break;
        }

        case sysAppLaunchCmdNotify: {
            HandleNotification( (SysNotifyParamType *)cmdPBP );
            break;
        }

        default:
            break;
    }
    return 0;
}

